#ifndef GRAPH_H_
#define GRAPH_H_
#include <iostream>
#include <vector>


namespace AG{

  // a node id is just the index in a vector
  typedef unsigned int vertex;

  template <typename Tvertex, typename Tedge>
  class graph{

  public:
  
    // vector of nodes label, the position in the vector is the id of the node
    typedef std::vector<Tvertex> Nodes; 
    typedef typename std::vector<Tvertex>::const_iterator Nodes_const_iterator;
    typedef typename std::vector<Tvertex>::iterator Nodes_iterator;
    
    // vector of edges (i, j, label_ij)
    //    i is the position in the vector -- here is fixed
    //       j is first in the pair
    //       label_ij is the second in the pair
    typedef std::pair<vertex, Tedge> arc_out_from_a_node;
    typedef std::vector<arc_out_from_a_node> arcs_out_from_a_node;
    typedef typename std::vector<arc_out_from_a_node>::const_iterator arcs_out_from_a_node_const_iterator;
    typedef typename std::vector<arc_out_from_a_node>::iterator arcs_out_from_a_node_iterator;

    // here i is changing and covers all the nodes in the graph
    //
    typedef std::vector<arcs_out_from_a_node> Edges;
    typedef typename std::vector<arcs_out_from_a_node>::const_iterator Edges_const_iterator;
    typedef typename std::vector<arcs_out_from_a_node>::iterator Edges_iterator;


  private:
    Nodes nodes_; // a vector of nodes
    Edges edges_; // vector of vectors of egdes
    unsigned int num_nodes_;
  public:
    
    // constructor destructors

    graph(){};
    graph(unsigned int n) : num_nodes_(n) 
    { 
      nodes_.resize(n); 
      edges_.resize(n);   // assume outdegree of 2, this may be enlarged
    }
    ~graph(){}

    // numNodes
    inline unsigned int numNodes() const { return num_nodes_; }

    // begin end
    inline Nodes_iterator begin() { return nodes_.begin(); }
    inline Nodes_iterator end() { return nodes_.end(); }

    // insert is op []
    Tvertex& operator[] (const unsigned int i){ return nodes_.at(i); }; 

    // insert a new node (dynamic graph)
    void push_back(Tvertex nlabel) 
    { 
      arcs_out_from_a_node e;
      nodes_.push_back(nlabel); // insert the new vertex with its label
      edges_.push_back(e);      // empty arcs exiting from this node -- here
      ++num_nodes_;
    }

    // insert a new outstar from a node
    void insertEdges(vertex v, arcs_out_from_a_node& a)
    {
      edges_.at(v) = a;
    }
    
    // return the outstar of a noce
    arcs_out_from_a_node& getEdges(vertex v)
    {
      try {
	arcs_out_from_a_node& a = edges_.at(v);
	return a;
      } catch (...){
	throw;
      }
      // will never reach this point; but make the compiler not complain
      arcs_out_from_a_node& a = edges_.at(0);
      return a;
    }

    friend std::ostream& operator<< (std::ostream& os, graph& g)
    {
      Nodes_const_iterator nit = g.nodes_.begin();
      Nodes_const_iterator nitend = g.nodes_.end();

      Edges_const_iterator eit = g.edges_.begin();
      Edges_const_iterator eitend = g.edges_.end();      

      arcs_out_from_a_node_const_iterator aeit, aeitend;

      for (vertex id=0; nit != nitend && eit != eitend; 
	   ++id, ++nit, ++eit){
	os << "Node ["<< id << "]= " << *nit << " " <<std::endl;

	aeitend = eit->end();   // scan all the outstar of this node
	for (aeit = eit->begin(); aeit != aeitend; ++aeit)
	  os << "\t(" << id << "," << aeit->first << ")=" 
	     << aeit->second << std::endl;
      }
      return os;
    }; 
  };

  //  #include "graph.cpp"

};

#endif
