#ifndef SEARCH_GRAPH_H
#define SEARCH_GRAPH_H
#include <iostream>
#include <vector>
#include <queue>
#include "graph.hpp"

namespace AG{

  typedef std::vector<vertex> Component;         // a component 
  typedef std::vector<vertex>::const_iterator Component_const_iterator; 
  typedef std::vector<Component> Components;    // multiple components
  typedef std::vector<Component>::const_iterator Components_const_iterator;
  
  template <class Graph>
  inline void internalsearch_(Graph& g, vertex currId,  
			      std::vector<bool>& visited,
			      Component& component)
  {
    std::cerr << "Visiting node " << currId << std::endl;
    
    try {
      typename Graph::arcs_out_from_a_node & a = 
	g.getEdges(currId);    
      
      visited[currId] = true;
      component.push_back(currId);
      
      typename Graph::arcs_out_from_a_node_iterator it, itend;
      itend = a.end();
      for (it = a.begin();it != itend; ++it)
	if (!visited[it->first])
	  internalsearch_(g, it->first, visited, component);    
      
    }catch (...)
      { std::cout << "\tnode="<< currId << " not in the graph" << std::endl;}
  }

  template<class Graph>
  void depth_search(Graph& g,  Components& components){

    unsigned int curr_component = 0;

    vertex maxId = g.numNodes();
    std::vector<bool> visited(maxId, false);

    for (vertex currId = 0; currId < maxId; ++currId)
      if (!visited[currId]){
	
	Component a_component;
	components.push_back(a_component);
	internalsearch_(g, currId, visited, components[curr_component++]);
      }
  }
  
  template <class Graph>
  inline void breath_search(Graph& g, 
			    vertex start, vertex end){

    std::cout << "Called breath search" << std::endl;

    std::queue<vertex> Q;
    std::vector<int> parent(g.numNodes(), -1);

    Q.push(start);
    parent[start] = start;

    while (!Q.empty()){
      
      vertex i = Q.front(); 
      Q.pop();

      std::cout << "Visiting Node= " << i << std::endl;

      try {
	typename Graph::arcs_out_from_a_node & a = 
	  g.getEdges(i);    
	typename Graph::arcs_out_from_a_node_iterator it, itend;
	itend = a.end();
	for (it = a.begin();it != itend; ++it){

	  if (parent[it->first] == -1){  // node not visited
	    parent[it->first] = i;       // (i, it->first) is edge
	    Q.push(it->first);           // remember to go there
	  }
	}

      }catch (...)
	{ std::cout << "\tnode="<< i << " not in the graph" << std::endl;}

    }
  };
};

#endif
