#include "graph.hpp"  // generic graph
#include "search.hpp" // generic search on graph

using namespace AG;

int main(){

  typedef int vertex_label;
  typedef int edge_label;

  static unsigned int number_nodes=10;

  graph<vertex_label, edge_label> g(number_nodes);
  graph<vertex_label, edge_label>::arcs_out_from_a_node a1, a3;
  graph<vertex_label, edge_label>::arcs_out_from_a_node_const_iterator 
    ait1, ait2;
  
  for (unsigned int node = 0; node < number_nodes; node++)
    g[node] = node;  // assign label/weight i to node i

  g[0]=1;   // assign label/weight 1 to node 0

  
  // dynamically add a new node to the graph
  g.push_back(13); 

  a1.push_back(std::make_pair(2, 40));
  a1.push_back(std::make_pair(3, 50));
  a1.push_back(std::make_pair(6, 50));
  a1.push_back(std::make_pair(2, 30));  // multiple egdes among 1-2
  g.insertEdges(1, a1);
  a3.push_back(std::make_pair(5, 40));
  a3.push_back(std::make_pair(3, 20));
  a3.push_back(std::make_pair(7, 100));
  a3.push_back(std::make_pair(1, 20));  // multiple egdes among 1-2
  g.insertEdges(3, a3);


  // If i get a reference I can dynamically add new edges to the node
  graph<vertex_label, edge_label>::arcs_out_from_a_node & a2 = 
    g.getEdges(1);
  a2.push_back(std::make_pair(69, 1969));

  ait2 = a2.end();
  std::cout << "Edges for node 1"<<std::endl;
  for (ait1 = a2.begin(); ait1 != ait2; ++ait1)
    std::cout << "(1," << ait1->first << ")=" << ait1->second << std::endl;
  
  
  std::cout << "Dump graph" << std::endl;
  
  std::cout << g;

  //graph<vertex_label, edge_label>::Nodes order;
  //depth_search(g,order);
  
  Components components;
  depth_search(g, components);

  Components_const_iterator csit, csit_end;
  Component_const_iterator cit, cit_end;

  csit_end = components.end();
  unsigned int currComponent = 0;
  for (csit = components.begin(); csit != csit_end; ++csit, ++currComponent){
    
    std::cout << "Component " << currComponent << std::endl;
    cit_end = csit->end();
    for (cit = csit->begin(); cit != cit_end; ++cit)
      std::cout << "\t Node=" << *cit << std::endl;
  }

  breath_search(g, 1, g.numNodes());

}
